﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using C1CollectionView101.Resources;
using System.Collections.ObjectModel;
using C1.CollectionView;
using C1.Win.CollectionView;
using System.Reflection;

namespace C1CollectionView101.View
{
    public partial class Grouping : UserControl
    {
        Menu _owner;
        private ObservableCollection<YouTubeVideo> _videos;
        private C1CollectionView<YouTubeVideo> _collectionView;

        public Grouping()
        {
            InitializeComponent();
            lblTitle.Text = "Grouping, Sorting and Filtering using C1CollectionView"; 
            listView1.DoubleBuffering(true);
        }

        internal async void ShowPage(Menu owner)
        {
            _owner = owner;
            await UpdateVideos();
        }
        private async Task UpdateVideos()
        {
            try
            {
                listView1.Visible = false;
                progressBar1.Visible = true;

                _videos = new ObservableCollection<YouTubeVideo>((await YouTubeCollectionView.LoadVideosAsync("WinForms", "relevance", null, 50)).Item2);
                _collectionView = new C1CollectionView<YouTubeVideo>(_videos);

                //Group C1CollectionView
                await _collectionView.GroupAsync();
                await _collectionView.GroupAsync(v => v.ChannelTitle);

                ////Sort C1CollectionView
                //var direction = GetCurrentSortDirection();
                //await _collectionView.SortAsync(v => v.ChannelTitle, direction == SortDirection.Ascending ? SortDirection.Descending : SortDirection.Ascending);

                listView1.SetItemsSource(_collectionView, "Title", "Thumbnail");

                _collectionView.SortChanged += OnSortChanged;
                UpdateSortButton();

                listView1.Visible = true;
            }
            catch
            {
                MessageBox.Show(AppResources.InternetConnectionError, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                progressBar1.Visible = false;
            }
        }
        private void UpdateSortButton()
        {
            var direction = GetCurrentSortDirection();
            if (direction == SortDirection.Ascending)
            {
                btnSort.ImageKey = "sort-descending.png";
            }
            else
            {
                btnSort.ImageKey = "sort-ascending.png";
            }
        }
        void OnSortChanged(object sender, EventArgs e)
        {
            UpdateSortButton();
        }
        private void btnBack_Click(object sender, EventArgs e)
        {
            _owner.SelectedSampleViewType = -1;
        }

        private async void filterTextBox_TextChanged(object sender, EventArgs e)
        {
            progressBar1.Visible = true;
            //Filter C1CollectionView
            await _collectionView.FilterAsync(filterTextBox.Text);
            progressBar1.Visible = false;

        }
        private SortDirection GetCurrentSortDirection()
        {
            var sort = _collectionView.SortDescriptions.FirstOrDefault(sd => sd.SortPath == "ChannelTitle");
            return sort != null ? sort.Direction : SortDirection.Descending;
        }

        async void btnSort_Click(object sender, EventArgs e)
        {
            if (_collectionView != null)
            {
                var direction = GetCurrentSortDirection();
                await _collectionView.SortAsync(x => x.ChannelTitle, direction == SortDirection.Ascending ? SortDirection.Descending : SortDirection.Ascending);
            }
        }
    }
    public static class ControlExtension
    {
        public static void DoubleBuffering(this Control control, bool enable)
        {
            var method = typeof(Control).GetMethod("SetStyle", BindingFlags.Instance | BindingFlags.NonPublic);
            method.Invoke(control, new object[] { ControlStyles.OptimizedDoubleBuffer, enable });
        }
    }
}
